clearvars;
close all;
clc;

%% set parameters

% wage elasticities
gamma = -0.5;
rho = 0.4; % conservative dtc elasticity
rhostrong = 1; % strong bias dtc elasticity
rhoexw = -gamma; % dtc elasticity to create exogenous wage benchmark
sigma = -1/gamma;
delta = (1+gamma)^2/rho+1+gamma;
deltastrong = (1+gamma)^2/rhostrong+1+gamma;
deltaexw = (1+gamma)^2/rhoexw+1+gamma;

% current tax function (CRP approx of US 2005 taxes, from Heathcote,
% Storesletten and Violante 2017)
p = 0.181; % rate of progressivity
lambda = 5.568;

% exogenous government revenue requirement, in percent of 2005 GDP
revexrel = 0.0; % baseline: 0

% labor supply elasticity
e = 0.5; % elasticity around linearized budget set, range: 0.27 to 0.64, baseline: 0.5
er = e/(1+e*p); % elasticity wrt marginal retention rate
ew = (1-p)*e/(1+e*p); % elasticity wrt wage
ehigh = 0.64; % high elasticity case for tax reform analysis
erhigh = ehigh/(1+ehigh*p); % elasticity wrt retention rate around actual budget set
ewhigh = (1-p)*ehigh/(1+ehigh*p); % elasticity wrt wage around actual budget set

% current income distribution
mu = 10.6; % mean of lognormal part
sigmay = 0.85; % variance of lognormal part
a = 1.5; % Pareto tail parameter
yth1 = 200000; % threshold for merging lognormal and Pareto

smooth = 75000; % for smoothing of the current income hazard ratio

mass = 0.02; % mass of non-working agents, baseline: 0.02

% income sample
spac1 = 1000; % spacing in first interval
spac2 = 10000; % spacing in second interval
yth2 = 350000; % second income threshold (see above for the first)
spac3 = 50000; % spacing in third interval
yth3 = 600000; % third income threshold
spac4 = 200000; % spacing in fouth interval
yth4 = 40000000; % upper bound

%% Compute pdf and cdf of current income distribution

% construct domain (income sample)
y = [5000:spac1:yth1, (yth1+spac2):spac2:yth2, (yth2+spac3):spac3:yth3, (yth3+spac4):spac4:yth4];

% get index for threshold 3 and use threshold 3 as the upper bound for
% plots
indyth3 = find((y==yth3));
yplot = y(1:indyth3);

% construct hazard ratio of the income distribution
hr = zeros(1,length(y));
for i = 1:length(y)
    hr(i) = hazardratiosmooth(y(i),mu,sigmay,a,yth1,smooth);
end
%hr(1) = lognpdf(1,mu,sigmay)/(1-logncdf(1,mu,sigmay)); % to avoid hr(1)=0

% plot hazard ratio
hrplot = hr(1:indyth3); % hazard ratio in plot range
figure('Units','centimeters','Position',[2 1 16 9])
plot(yplot,hrplot,'k');
xlim([0 600000]);
xticks([0 50000 100000 200000 300000 400000 500000 600000]);
xticklabels({'0','50','100','200','300','400','500','600'});
xlabel('Current Income in $1000');
ylim([0 3])
yticks([0 0.5 1 1.5 2 2.5 3]);
ylabel('Hazard Ratio');
legend('hide');
%matlab2tikz('hazardratio.tex'); % uncomment for tikz export

% compute cdf corresponding to the hazard ratio
[y,My] = ode23(@(x,My) hazardratiosmooth(x,mu,sigmay,a,yth1,smooth)/x - ...
    hazardratiosmooth(x,mu,sigmay,a,yth1,smooth)/x*My,y,0);

Myplot = My(1:indyth3); % cdf in the plot range
% plot(yplot,Myplot); % uncomment to plot cdf

% compute pdf corresponding to the cdf
my = gradient(My,y);

myplot = my(1:indyth3); % pdf in plot range
% plot(yplot,myplot); % uncomment to plot pdf

% check average income and max hazard ratio, matched to US 2005 earnings
% distribution (see, e.g., Figure 2 in Diamond and Saez 2011)
ymean = trapz(y,y.*my);
hrmax = max(hr);

% compute absolute government spending requirement
revex = ymean*revexrel;

%% compute exogenous technology component kappa

% compute current labor inputs
l = (lambda*(1-p))^(e/(1+e))*y.^((1-p)*e/(1+e));

% compute kappa
kappa = y.^(1/(1+gamma+rho)).*l.^(-1)*ymean^(gamma/(1+gamma+rho)); % conservative
kappastrong = y.^(1/(1+gamma+rhostrong)).*l.^(-1)*ymean^(gamma/(1+gamma+rhostrong)); % strong bias
kappaexw = y.^(1/(1+gamma+rhoexw)).*l.^(-1)*ymean^(gamma/(1+gamma+rhoexw)); % exogenous wages

% compute R&D resource constraint \overline{C}
cbar = trapz(My,(kappa.*l).^((sigma-1)*delta/((delta-1)*sigma+1))); % conservative
cbarstrong = trapz(My,(kappastrong.*l).^((sigma-1)*deltastrong/((deltastrong-1)*sigma+1))); % strong bias
cbarexw = trapz(My,(kappaexw.*l).^((sigma-1)*deltaexw/((deltaexw-1)*sigma+1))); % exogenous wages

% compute equilibrium technology at current taxes
phistar = (kappa.*l).^(rho/(1+gamma)); % conservative
%phistartest = y.^(rho/((1+gamma+rho)*(1+gamma)))*ymean^(gamma*rho/((1+gamma+rho)*(1+gamma)));
phistarstrong = (kappastrong.*l).^(rhostrong/(1+gamma)); % strong bias
%phistarstrongtest = y.^(rhostrong/((1+gamma+rhostrong)*(1+gamma)))*ymean^(gamma*rhostrong/((1+gamma+rhostrong)*(1+gamma)));
phistarexw = (kappaexw.*l).^(rhoexw/(1+gamma)); % exogenous wages

% compute wages
w = y./l;

% check average income: average income equals total output
ymeancheck = trapz(My,(kappa.*l).^(1+gamma+rho))^(1/(1+gamma));
ymeancheckstrong = trapz(My,(kappastrong.*l).^(1+gamma+rhostrong))^(1/(1+gamma));

% compute government revenue
ytax = y-lambda*y.^(1-p); % individual tax payments
stotal = trapz(My,ytax); % total tax payments
% compute lump sum transfer
s = 1/(1+mass)*(stotal-revex); % government surplus divided by mass of agents

% compute current marginal and average tax rates (for comparison with the
% optimal tax)
mtrcurrent = 1-lambda*(1-p)*y.^(-p);
atrcurrent = (ytax-s)./y;

%% simulation 1: compute wage effects of taking tax progressivity back to its 1970 value

% Set parameters
p70 = p+0.034; % progressivity of US taxes in 1970-76 (from Heathcote et al. (2017))

% Compute post-reform value of lambda such that gov revenue is unchanged in
% the conservative case (rho = 0.4, gamma = -0.5)
lambda70 = fsolve(@(lambdanew)revenuechange(lambdanew,p70,lambda,p,My,y,ymean,er,ew,gamma,rho),lambda); % baseline labor supply elasticity
lambda70high = fsolve(@(lambdanew)revenuechange(lambdanew,p70,lambda,p,My,y,ymean,erhigh,ewhigh,gamma,rho),lambda); % high labor supply elasticity
% lambda70 = lambda;

% Compute relative wage change caused by the reform, based on the formulas
% provided in the paper (linear approximations)
[what,whatnodtc] = wagechange(lambda,lambda70,p,p70,My,y,ymean,er,ew,gamma,rho); % conservative and exo tech
whatstrong = wagechange(lambda,lambda70,p,p70,My,y,ymean,er,ew,gamma,rhostrong); % strong bias
[whathigh,whatnodtchigh] = wagechange(lambda,lambda70high,p,p70,My,y,ymean,erhigh,ewhigh,gamma,rho); % cons. and exo tech with high elasticity
whatstronghigh = wagechange(lambda,lambda70high,p,p70,My,y,ymean,erhigh,ewhigh,gamma,rhostrong); % strong bias with high elasticity

% plot, restrict wage changes to plot range
whatplot = what(1:indyth3);
whatnodtcplot = whatnodtc(1:indyth3);
whatstrongplot = whatstrong(1:indyth3);
whathighplot = whathigh(1:indyth3);
whatnodtchighplot = whatnodtchigh(1:indyth3);
whatstronghighplot = whatstronghigh(1:indyth3);

% plot for baseline labor supply elasticity
figure('Units','centimeters','Position',[2 1 16 9])
plot(Myplot,whatstrongplot,'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on;
plot(Myplot,whatplot,'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(Myplot,whatnodtcplot,'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
title('Low Labor Supply Elasticity');
xlim([0 1]);
xticks([0 0.2 0.4 0.6 0.8 1]);
xlabel('Type Measured by Percentile of the Income Distribution');
ylim([-0.05 0.05]);
yticks([-0.05 -0.04 -0.03 -0.02 -0.01 0 0.01 0.02 0.03 0.04 0.5]);
yticklabels({'','-4','','-2','','0','','2','','4',''});
% ylim([-0.1 0.1]);
% yticks([-0.1 -0.075 -0.05 -0.025 0 0.025 0.05 0.075 0.1]);
% yticklabels({'-10','','-5','','0','','5','','10'});
ylabel('Wage Change in %');
legend('Location','northeast');
legend('boxoff');
zline1 = line(xlim, [0 0],'Color','black');
zline1.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('wagechange1.tex');

% plot for high labor supply elasticity
figure
plot(Myplot,whatstronghighplot,'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on;
plot(Myplot,whathighplot,'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(Myplot,whatnodtchighplot,'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
title('High Labor Supply Elasticity');
xlim([0 1]);
xticks([0 0.2 0.4 0.6 0.8 1]);
xlabel('Type Measured by Percentile of the Income Distribution');
ylim([-0.05 0.05]);
yticks([-0.05 -0.04 -0.03 -0.02 -0.01 0 0.01 0.02 0.03 0.04 0.5]);
yticklabels({'','-4','','-2','','0','','2','','4',''});
% ylim([-0.1 0.1])
% yticks([-0.1 -0.075 -0.05 -0.025 0 0.025 0.05 0.075 0.1]);
%yticklabels({'-10','','-5','','0','','5','','10'});
ylabel('Wage Change in %');
legend('Location','northeast');
legend('boxoff');
zline2 = line(xlim, [0 0],'Color','black');
zline2.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('wagechange2.tex');

% compute change in 90-10 percentile ratio due to the tax reform
yperc10 = find((Myplot>=0.1),1); % index of 10 perc
yperc90 = find((Myplot>=0.9),1); % index of 90 perc
w9010change = what(yperc90)-what(yperc10); % conservative
w9010changestrong = whatstrong(yperc90)-whatstrong(yperc10); % strong bias
w9010changehigh = whathigh(yperc90)-whathigh(yperc10); % conservative, high ls elasticity
w9010changestronghigh = whatstronghigh(yperc90)-whatstronghigh(yperc10); % strong bias, high ls elasticity

% for comparison: wage effects without linearization
% only for baseline e
% results for ehigh: set e to ehigh
% joint computation requires case distinction in calibration, which seems
% unnecessary given the tiny difference between linearized and
% non-linearized effects for baseline e
% compute labor inputs under 1970 taxes
l70 = laborinputcrp(p70,lambda70,My,kappa,cbar,e,gamma,rho,delta); % computes labor inputs under 1970 tax
l70strong = laborinputcrp(p70,lambda70,My,kappastrong,cbarstrong,e,gamma,rhostrong,deltastrong); % same for strong bias

% compute wages under 1970 taxes
w70 = ((1-p70)*lambda70)^(-1/(1-p70))*l70.^((1+e*p70)/(e*(1-p70)));
w70strong = ((1-p70)*lambda70)^(-1/(1-p70))*l70strong.^((1+e*p70)/(e*(1-p70)));

% compute relative wage change when moving from 2005 to 1970 taxes
whatnl = w70./w-1; % conservative case, not based on linear approx
whatstrongnl = w70strong./w-1; % strong bias case

% compute changes in 90 to 10 percentile ratio
w9010changenl = whatnl(yperc90)-whatnl(yperc10);
w9010changestrongnl = whatstrongnl(yperc90)-whatstrongnl(yperc10);

% plot, restrict to plot range
whatnlplot = whatnl(1:indyth3);
whatstrongnlplot = whatstrongnl(1:indyth3);

figure
plot(Myplot,whatstrongnlplot,'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on;
plot(Myplot,whatnlplot,'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
title('Low Labor Supply Elasticity');
xlim([0 1]);
xticks([0 0.2 0.4 0.6 0.8 1]);
xlabel('Type Measured by Percentile of the Income Distribution');
ylim([-0.05 0.05]);
yticks([-0.05 -0.04 -0.03 -0.02 -0.01 0 0.01 0.02 0.03 0.04 0.5]);
yticklabels({'','-4','','-2','','0','','2','','4',''});
% ylim([-0.1 0.1]);
% yticks([-0.1 -0.075 -0.05 -0.025 0 0.025 0.05 0.075 0.1]);
% yticklabels({'-10','','-5','','0','','5','','10'});
ylabel('Wage Change in %');
legend('Location','northeast');
legend('boxoff');
zline1 = line(xlim, [0 0],'Color','black');
zline1.Annotation.LegendInformation.IconDisplayStyle = 'off';

% check convergence conditions from Lemma 3:
concheck1 = (ew*rho)^2*(1+sqrt(trapz(My,(y/ymean).^2)))^2;
concheck1strong = (ew*rhostrong)^2*(1+sqrt(trapz(My,(y/ymean).^2)))^2;
concheck2 = (ew*(rho+gamma))^2*(1+sqrt(trapz(My,(y/ymean).^2)))^2;
concheck2strong = (ew*(rhostrong+gamma))^2*(1+sqrt(trapz(My,(y/ymean).^2)))^2;

%% simulation 2: compute welfare effects of taking tax progressivity back to its 1970 value

% set range for relative inequality aversion parameter
rrange = (0.1:0.1:50);

% compute 2005 consumption and utilities
c = lambda*y.^(1-p) + s;
u = c - e/(e+1)*l.^((e+1)/e);

% compute 1970 consumption
y70 = w70.*l70; %1970 income
y70strong = w70strong.*l70strong;
ytax70 = y70-lambda70*y70.^(1-p70); %1970 tax payments
ytax70strong = y70strong-lambda70*y70strong.^(1-p70);
stotal70 = trapz(My,ytax70); %1970 revenue
stotal70strong = trapz(My,ytax70strong);
s70 = 1/(1+mass)*(stotal70-revex); %1970 transfer
s70strong = 1/(1+mass)*(stotal70strong-revex);
c70 = lambda70*y70.^(1-p70) + s70; %1970 consumption
c70strong = lambda70*y70strong.^(1-p70) + s70strong;

%preallocation
welfarehat = zeros(1,numel(rrange));
welfarehatnodtc = zeros(1,numel(rrange));
welfarehatstrong = zeros(1,numel(rrange));
cwelfarediffabs = zeros(1,numel(rrange));
cwelfarediffnodtcabs = zeros(1,numel(rrange));
cwelfarediffstrongabs = zeros(1,numel(rrange));
cwelfarediffgdp = zeros(1,numel(rrange));
cwelfarediffnodtcgdp = zeros(1,numel(rrange));
cwelfarediffstronggdp = zeros(1,numel(rrange));
cwelfarediffabsnl = zeros(1,numel(rrange));
cwelfarediffstrongabsnl = zeros(1,numel(rrange));
cwelfarediffgdpnl = zeros(1,numel(rrange));
cwelfarediffstronggdpnl = zeros(1,numel(rrange));

for i = 1:numel(rrange)
    r = rrange(i);
    
    %construct welfare weights
    g = u.^(-r)/(mass*s^(-r)+trapz(My,u.^(-r)));
    
    % compute 2005 welfare
    welfare05 = welfare(c,l,s,mass,r,My,e);
    
    % compute 1970 welfare
    welfare70 = welfare(c70,l70,s70,mass,r,My,e); % no need for scaling here
    welfare70strong = welfare(c70strong,l70strong,s70strong,mass,r,My,e);
    
    %construct scaling factor for welfare, such that partial derivatives
    %equal welfare weights
    welfarescale = 1/(welfare05^r*(mass*s^(-r)+trapz(My,u.^(-r))));

    % compute welfare changes, based on formulas provided in the paper
    % (linear approximations)
    [welfarehataux,welfarehatnodtcaux] = welfarechange(lambda,lambda70,p,p70,My,y,ymean,er,ew,gamma,rho,g,welfare05);
    welfarehat(i) = 1/welfarescale*welfarehataux; % conservative
    welfarehatnodtc(i) = 1/welfarescale*welfarehatnodtcaux; % exo tech
    welfarehatstrong(i) = 1/welfarescale*welfarechange(lambda,lambda70,p,p70,My,y,ymean,er,ew,gamma,rhostrong,g,welfare05); % strong bias
    
    % welfare-equivalent consumption changes
    % linearized
    % compute change in lump sum payment that produces welfare change
    % equivalent to tax reform
    cwelfarediffabs(i) = fsolve(@(add) 100000*(welfare(c+add,l,s+add,mass,r,My,e)-welfare05*(1+welfarehat(i))),1); % conservative
    cwelfarediffnodtcabs(i) = fsolve(@(add) 100000*(welfare(c+add,l,s+add,mass,r,My,e)-welfare05*(1+welfarehatnodtc(i))),1); % exo tech
    cwelfarediffstrongabs(i) = fsolve(@(add) 100000*(welfare(c+add,l,s+add,mass,r,My,e)-welfare05*(1+welfarehatstrong(i))),1); % strong bias
    cwelfarediffgdp(i) = cwelfarediffabs(i)/ymean; % as percent of pre-reform average income, conservative
    cwelfarediffnodtcgdp(i) = cwelfarediffnodtcabs(i)/ymean; % same for exo tech
    cwelfarediffstronggdp(i) = cwelfarediffstrongabs(i)/ymean; % same for strong bias
    % non-linearized, same as above but for exact changes
    cwelfarediffabsnl(i) = fsolve(@(add) 100000*(welfare(c+add,l,s+add,mass,r,My,e)-welfare70),1); % conservative
    cwelfarediffstrongabsnl(i) = fsolve(@(add) 100000*(welfare(c+add,l,s+add,mass,r,My,e)-welfare70strong),1); % strong bias
    cwelfarediffgdpnl(i) = cwelfarediffabsnl(i)/ymean; % relative to average income
    cwelfarediffstronggdpnl(i) = cwelfarediffstrongabsnl(i)/ymean; % same for strong bias
end

% plot relative change of the welfare function (no consumption
% equivalents), linearized
figure
plot(rrange,welfarehatstrong,'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(rrange,welfarehat,'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(rrange,welfarehatnodtc,'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
xlim([0 50]);
xticks([0 10 20 30 40 50]);
xlabel('Relative Inequality Aversion')
ylim([-0.01 0.015]);
yticks([-0.01 -0.005 0 0.005 0.01 0.015]);
yticklabels({'-1','','0','','1',''});
ylabel('Welfare-Equivalent Consumption Change in % of Average Income')
legend('Location','northeast');
legend('boxoff');
zline3 = line(xlim,[0 0],'Color','black');
zline3.Annotation.LegendInformation.IconDisplayStyle = 'off';
cleanfigure;
%matlab2tikz('welfarechange.tex');

% plot welfare changes in consumption equivalents, linearized
figure('Units','centimeters','Position',[2 1 16 9])
plot(rrange,cwelfarediffstronggdp,'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(rrange,cwelfarediffgdp,'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(rrange,cwelfarediffnodtcgdp,'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
xlim([0 50]);
xticks([0 10 20 30 40 50]);
xlabel('Relative Inequality Aversion');
ylim([-0.01 0.01]);
yticks([-0.01 -0.005 0 0.005 0.01]);
yticklabels({'-1','-0.5','0','0.5','1'});
ylabel('Welfare-Equivalent Consumption Change in % of Average Income')
legend('Location','northeast');
legend('boxoff');
zline3 = line(xlim,[0 0],'Color','black');
zline3.Annotation.LegendInformation.IconDisplayStyle = 'off';
cleanfigure;
%matlab2tikz('welfarechange.tex');

% plot exact (non-linearized) welfare changes in consumption equivalents
figure
plot(rrange,cwelfarediffstronggdpnl,'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(rrange,cwelfarediffgdpnl,'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
xlim([0 50]);
xticks([0 10 20 30 40 50]);
xlabel('Relative Inequality Aversion');
ylim([-0.01 0.01]);
yticks([-0.01 -0.005 0 0.005 0.01]);
yticklabels({'-1','-0.5','0','0.5','1'});
ylabel('Welfare-Equivalent Consumption Change in % of Average Income')
legend('Location','northeast');
legend('boxoff');
zline3 = line(xlim,[0 0],'Color','black');
zline3.Annotation.LegendInformation.IconDisplayStyle = 'off';
cleanfigure;

 %% simulation 3: optimal taxes

% compute inverse hazard ratio of beta
% conservative:
betainvhr = (1./hr*(1-(gamma+rho)*ew)/(1+ew))';
beta = kappa.^(1+gamma+rho);
% strong bias:
betainvhrstrong = (1./hr*(1-(gamma+rhostrong)*ew)/(1+ew))';
betastrong = kappastrong.^(1+gamma+rhostrong);
% exogenous wages:
betainvhrexw = (1./hr*(1-(gamma+rhoexw)*ew)/(1+ew))';
betaexw = kappaexw.^(1+gamma+rhoexw);

% compute inverse hazard ratio of \overline{\beta} (for exo tech planner)
betabarinvhr = (1./hr*(1-gamma*ew)/(1+ew))';
betabar = kappa.^(1+gamma).*phistar.^(1+gamma); % no need to distinguish between rho and rhostrong here, yields the same result

% set relative inequality aversion parameter
rrangeopt = [1,50]; % baseline values: 0.7 for e=0.4; 1 for e=0.5

% preallocation of result vectors
mtropt = zeros(numel(rrangeopt),numel(y));
mtroptex = zeros(numel(rrangeopt),numel(y));
mtroptstrong = zeros(numel(rrangeopt),numel(y));
mtroptexw = zeros(numel(rrangeopt),numel(y));

atropt = zeros(numel(rrangeopt),numel(y));
atroptex = zeros(numel(rrangeopt),numel(y));
atroptstrong = zeros(numel(rrangeopt),numel(y));
atroptexw = zeros(numel(rrangeopt),numel(y));

yopt = zeros(numel(rrangeopt),numel(y));
yoptex = zeros(numel(rrangeopt),numel(y));
yoptstrong = zeros(numel(rrangeopt),numel(y));
yoptexw = zeros(numel(rrangeopt),numel(y));

welfareopt = zeros(numel(rrangeopt),1);
welfareoptstrong = zeros(numel(rrangeopt),1);
welfareoptexw = zeros(numel(rrangeopt),1);

efmono = zeros(numel(rrangeopt),1);
efmonoex = zeros(numel(rrangeopt),1);
efmonostrong = zeros(numel(rrangeopt),1);
efmonoexw = zeros(numel(rrangeopt),1);

efcounter = zeros(numel(rrangeopt),1);
efcounterex = zeros(numel(rrangeopt),1);
efcounterstrong = zeros(numel(rrangeopt),1);
efcounterexw = zeros(numel(rrangeopt),1);

for i = 1:numel(rrangeopt)
    % set inequality aversion
    r = rrangeopt(i);
    
    % compute optimal mtrs and corresponding income vectors
    % mtropt - optimal mtrs; yopt - income vector at optimum; welfareopt -
    % welfare level at optimum; efmono - test of monotonicity condition
    % (1-satisfied; 0-not satisfied); efcounter - test for convergence of
    % algorithm (1-converged; 0-not converged)
    % conservative case:
    [mtropt(i,:),atropt(i,:),yopt(i,:),welfareopt(i),efmono(i),efcounter(i)] = mtroptimal(My,mass,r,kappa,beta,betainvhr,cbar,e,gamma,rho,delta,revex);
    % exogenous tech planner:
    [mtroptex(i,:),atroptex(i,:),yoptex(i,:),efmonoex(i),efcounterex(i)] = mtroptimalex(My,mass,r,kappa,betabar,betabarinvhr,phistar,e,gamma,revex);
    % strong bias case:
    [mtroptstrong(i,:),atroptstrong(i,:),yoptstrong(i,:),welfareoptstrong(i),efmonostrong(i),efcounterstrong(i)] = mtroptimal(My,mass,r,kappastrong,betastrong,betainvhrstrong,cbarstrong,e,gamma,rhostrong,deltastrong,revex);
    % exogenous wage case:
    [mtroptexw(i,:),atroptexw(i,:),yoptexw(i,:),welfareoptexw(i),efmonoexw(i),efcounterexw(i)] = mtroptimal(My,mass,r,kappaexw,betaexw,betainvhrexw,cbarexw,e,gamma,rhoexw,deltaexw,revex);
end

% compute (asymptotic) optimal marginal top tax rates, based on formula
% provided in the paper
toptax = (1/(a*e)+(a-1)/a*(gamma+rho))/(1+1/(a*e)+(a-1)/a*(gamma+rho));
toptaxex = (1/(a*e)+(a-1)/a*gamma)/(1+1/(a*e)+(a-1)/a*gamma);
toptaxstrong = (1/(a*e)+(a-1)/a*(gamma+rhostrong))/(1+1/(a*e)+(a-1)/a*(gamma+rhostrong));
toptaxexw = (1/(a*e)+(a-1)/a*(gamma+rhoexw))/(1+1/(a*e)+(a-1)/a*(gamma+rhoexw));

% set current median income to provide orientation in figure
ymedian = 52700; % household median earnings in 2005, from US Census Bureau

% plots for optimal taxes, restrict to plot range
mtroptplot = mtropt(:,1:indyth3);
mtroptexplot = mtroptex(:,1:indyth3);
mtroptstrongplot = mtroptstrong(:,1:indyth3);
mtroptexwplot = mtroptexw(:,1:indyth3);

atroptplot = atropt(:,1:indyth3);
atroptexplot = atroptex(:,1:indyth3);
atroptstrongplot = atroptstrong(:,1:indyth3);
atroptexwplot = atroptexw(:,1:indyth3);

yoptplot = yopt(:,1:indyth3);
yoptexplot = yoptex(:,1:indyth3);
yoptstrongplot = yoptstrong(:,1:indyth3);
yoptexwplot = yoptexw(:,1:indyth3);

mtrcurrentplot = mtrcurrent(1:indyth3);
atrcurrentplot = atrcurrent(1:indyth3);

% optimal mtrs at current income, baseline
figure
plot(yplot,mtroptstrongplot(1,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yplot,mtroptplot(1,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yplot,mtroptexplot(1,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
title('Low Equity Concern');
xlim([0 500000]);
ylim([0.2 0.8]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Type Measured by Current Income in $1000');
yticks([0.2 0.3 0.4 0.5 0.6 0.7 0.8]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax1.tex');

% optimal mtrs at current income, baseline, with current tax system
figure
plot(yplot,mtroptstrongplot(1,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yplot,mtroptplot(1,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yplot,mtroptexplot(1,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
plot(yplot,mtrcurrentplot,'g','DisplayName','US 2005');
title('Low Equity Concern');
xlim([0 500000]);
ylim([0.2 0.8]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Type Measured by Current Income in $1000');
yticks([0.2 0.3 0.4 0.5 0.6 0.7 0.8]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax1cur.tex');

% optimal mtrs at income at the optimum, baseline
figure('Units','centimeters','Position',[2 1 16 9])
plot(yoptstrongplot(1,:),mtroptstrongplot(1,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yoptplot(1,:),mtroptplot(1,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yoptexplot(1,:),mtroptexplot(1,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
title('Low Equity Concern');
xlim([0 500000]);
ylim([0.2 0.8]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Income');
yticks([0.2 0.3 0.4 0.5 0.6 0.7 0.8]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax1alt.tex');

% optimal mtrs at income at the optimum, baseline, with exogenous wage
% benchmark
figure('Units','centimeters','Position',[2 1 16 9])
plot(yoptstrongplot(1,:),mtroptstrongplot(1,:),'--r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yoptplot(1,:),mtroptplot(1,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yoptexplot(1,:),mtroptexplot(1,:),'-.k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
plot(yoptexwplot(1,:),mtroptexwplot(1,:),':k','DisplayName','Exogenous Wage Planner');
title('Low Equity Concern');
xlim([0 500000]);
ylim([0.2 0.8]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Income');
yticks([0.2 0.3 0.4 0.5 0.6 0.7 0.8]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax1altexw.tex');

% optimal mtrs at income at the optimum, baseline, with current tax system
figure
plot(yoptstrongplot(1,:),mtroptstrongplot(1,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yoptplot(1,:),mtroptplot(1,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yoptexplot(1,:),mtroptexplot(1,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
plot(yplot,mtrcurrentplot,'g','DisplayName','US 2005');
title('Low Equity Concern');
xlim([0 500000]);
ylim([0.2 0.8]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Income');
yticks([0.2 0.3 0.4 0.5 0.6 0.7 0.8]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax1altcur.tex');

% optimal atr, at current income, baseline
figure
plot(yplot,atroptstrongplot(1,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yplot,atroptplot(1,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yplot,atroptexplot(1,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
title('Low Equity Concern');
xlim([0 500000]);
ylim([-0.6 0.8]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Type Measured by Current Income in $1000');
%yticks([0.4 0.5 0.6 0.7 0.8 0.9 1]);
ylabel('Average Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimalavetax1.tex');

% optimal mtr at current income, Rawlsian
figure
plot(yplot,mtroptstrongplot(2,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yplot,mtroptplot(2,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yplot,mtroptexplot(2,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
title('High Equity Concern');
xlim([0 500000]);
ylim([0.4 1]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Type Measured by Current Income in $1000');
yticks([0.4 0.5 0.6 0.7 0.8 0.9 1]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax2.tex');

% optimal mtr at current income, Rawlsian, with current tax
figure
plot(yplot,mtroptstrongplot(2,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yplot,mtroptplot(2,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yplot,mtroptexplot(2,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
plot(yplot,mtrcurrentplot,'g','DisplayName','US 2005');
title('High Equity Concern');
xlim([0 500000]);
ylim([0.4 1]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Type Measured by Current Income in $1000');
yticks([0.4 0.5 0.6 0.7 0.8 0.9 1]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax2cur.tex');

% optimal mtr at income at the optimum, Rawlsian, with current tax
figure
plot(yoptstrongplot(2,:),mtroptstrongplot(2,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yoptplot(2,:),mtroptplot(2,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yoptexplot(2,:),mtroptexplot(2,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
plot(yplot,mtrcurrentplot,'g','DisplayName','US 2005');
title('High Equity Concern');
xlim([0 500000]);
ylim([0.4 1]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Income');
yticks([0.4 0.5 0.6 0.7 0.8 0.9 1]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax2altcur.tex');

% optimal mtr at income at the optimum, Rawlsian
figure('Units','centimeters','Position',[2 1 16 9])
plot(yoptstrongplot(2,:),mtroptstrongplot(2,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yoptplot(2,:),mtroptplot(2,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yoptexplot(2,:),mtroptexplot(2,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
title('High Equity Concern');
xlim([0 500000]);
ylim([0.4 1]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Income');
yticks([0.4 0.5 0.6 0.7 0.8 0.9 1]);
ylabel('Marginal Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimaltax2alt.tex');

% optimal atr at current income, Rawlsian
figure
plot(yplot,atroptstrongplot(2,:),'r','DisplayName','\rho = 1 and \gamma = -0.5');
hold on
plot(yplot,atroptplot(2,:),'b','DisplayName','\rho = 0.4 and \gamma = -0.5');
plot(yplot,atroptexplot(2,:),'--k','DisplayName','Exogenous Technology Planner (\gamma = -0.5)');
title('High Equity Concern');
xlim([0 500000]);
ylim([-0.6 0.8]);
xticks([0 50000 100000 150000 200000 300000 400000 500000]);
xticklabels({'0','50','100','150','200','300','400','500'});
%xtickangle(45);
xlabel('Type Measured by Current Income in $1000');
%yticks([0.4 0.5 0.6 0.7 0.8 0.9 1]);
ylabel('Average Tax Rate');
legend('Location','northeast');
legend('boxoff');
medline = line([ymedian ymedian],ylim,'Color','black');
medline.Annotation.LegendInformation.IconDisplayStyle = 'off';
%matlab2tikz('optimalavetax2.tex');

% compute dtc impact on marginal tax rates

% maximum and minimum impact
% conservative:
mtrdiffmax = max(mtropt-mtroptex,[],2);
mtrdiffmin = min(mtropt-mtroptex,[],2);
% strong bias:
mtrdiffmaxstrong = max(mtroptstrong-mtroptex,[],2);
mtrdiffminstrong = min(mtroptstrong-mtroptex,[],2);

% impact at lowest and highest decile
[x10,ind10] = min(abs(My-0.1)); % index for 10 perc
[x90,ind90] = min(abs(My-0.9)); % index for 90 perc
% conservative:
mtrdiff10 = mtropt(:,ind10)-mtroptex(:,ind10);
mtrdiff90 = mtropt(:,ind90)-mtroptex(:,ind90);
% strong bias:
mtrdiff10strong = mtroptstrong(:,ind10)-mtroptex(:,ind10);
mtrdiff90strong = mtroptstrong(:,ind90)-mtroptex(:,ind90);

% impact on average marginal tax rate (not income-weighted)
mtraveopt = trapz(My,mtropt'); % conservative
mtraveoptex = trapz(My,mtroptex'); % exo tech
mtraveoptstrong = trapz(My,mtroptstrong'); % strong bias
mtravecurrent = trapz(My,mtrcurrent); % current

% impact on average marginal tax rate (income-weighted)
mtraveoptinc = trapz(My,mtropt'.*yopt')./trapz(My,yopt');
mtraveoptexinc = trapz(My,mtroptex'.*yoptex')./trapz(My,yoptex');
mtraveoptstronginc = trapz(My,mtroptstrong'.*yoptstrong')./trapz(My,yoptstrong');
mtravecurrentinc = trapz(My,mtrcurrent.*y)/ymean;

% compute dtc impact on average tax rates

% maximum and minimum impact
% conservative:
atrdiffmax = max(atropt-atroptex,[],2);
atrdiffmin = min(atropt-atroptex,[],2);
% strong bias:
atrdiffmaxstrong = max(atroptstrong-atroptex,[],2);
atrdiffminstrong = min(atroptstrong-atroptex,[],2);

% impact at lowest and highest decile
% conservative:
atrdiff10 = atropt(:,ind10)-atroptex(:,ind10);
atrdiff90 = atropt(:,ind90)-atroptex(:,ind90);
% strong bias:
atrdiff10strong = atroptstrong(:,ind10)-atroptex(:,ind10);
atrdiff90strong = atroptstrong(:,ind90)-atroptex(:,ind90);

% compute welfare gains from optimal taxes

% relative to current taxes
% frist: welfare under current taxes

c = lambda*y.^(1-p) + s; % consumption

% preallocation
welfarecurrent = zeros(numel(rrangeopt),1);
cwelfaregainlumpabs = zeros(numel(rrangeopt),1);
cwelfaregainlumpabsstrong = zeros(numel(rrangeopt),1);
cwelfaregainlumpgdp = zeros(numel(rrangeopt),1);
cwelfaregainlumpgdpstrong = zeros(numel(rrangeopt),1);
cwelfaregainlump = zeros(numel(rrangeopt),1);
cwelfaregainlumpstrong = zeros(numel(rrangeopt),1);

% compute welfare and welfare gains in consumption equivalents
for i = 1:numel(rrangeopt)
    r = rrangeopt(i);
    welfarecurrent(i) = welfare(c,l,s,mass,r,My,e); % welfare under current tax

    % compute lump sum payment equivalent to welfare gain from optimal tax
    % absolute change in lump sum payment:
    cwelfaregainlumpabs(i) = fsolve(@(add) 100000*(welfare(c+add,l,s+add,mass,r,My,e)-welfareopt(i)),1);
    cwelfaregainlumpabsstrong(i) = fsolve(@(add) 100000*(welfare(c+add,l,s+add,mass,r,My,e)-welfareoptstrong(i)),1);
    % relative to current average income:
    cwelfaregainlumpgdp(i) = cwelfaregainlumpabs(i)/ymean;
    cwelfaregainlumpgdpstrong(i) = cwelfaregainlumpabsstrong(i)/ymean;
    % relative to current lump sum payment:
    cwelfaregainlump (i) = cwelfaregainlumpabs(i)/s;
    cwelfaregainlumpstrong (i) = cwelfaregainlumpabsstrong(i)/s;
    
end

% relative to exogenous technology planner optimum

% compute welfare under exogenous tech planner optimum tax

% preallocation
welfareex = zeros(numel(rrangeopt),1);
welfareexstrong = zeros(numel(rrangeopt),1);
cwelfaregaintoexlumpabs = zeros(numel(rrangeopt),1);
cwelfaregaintoexlumpabsstrong = zeros(numel(rrangeopt),1);
cwelfaregaintoexlumpgdp = zeros(numel(rrangeopt),1);
cwelfaregaintoexlumpgdpstrong = zeros(numel(rrangeopt),1);
cwelfaregaintoexlump = zeros(numel(rrangeopt),1);
cwelfaregaintoexlumpstrong = zeros(numel(rrangeopt),1);

% compute welfare under exo tech optimum and welfare gains from optimal tax
% in consumption equivalents
for i = 1:numel(rrangeopt)
    r = rrangeopt(i);
    % compute income vector at exo tech mtrs
    % first compute labor supply, then obtain income via worker foc
    lex = laborinput(mtroptex(i,:)',My,kappa,beta,cbar,e,gamma,rho,delta);
    lexstrong = laborinput(mtroptex(i,:)',My,kappastrong,betastrong,cbarstrong,e,gamma,rhostrong,deltastrong);
    yex = lex.^((e+1)/e)./(1-mtroptex(i,:)');
    ymeanex = trapz(My,yex);
    yexstrong = lexstrong.^((e+1)/e)./(1-mtroptex(i,:)');
    ymeanexstrong = trapz(My,yexstrong);

    % compute tax liabilities without demogrant
    tex = cumtrapz(yex,mtroptex(i,:)');
    texstrong = cumtrapz(yexstrong,mtroptex(i,:)');

    % compute demogrant
    demoex = (revex-trapz(My,tex))*1/(1+mass);
    demoexstrong = (revex-trapz(My,texstrong))*1/(1+mass);

    % compute consumption and utility
    cex = yex - tex - demoex;
    cexstrong = yexstrong - texstrong - demoexstrong;

    % compute welfare
    welfareex(i) = welfare(cex,lex,-demoex,mass,r,My,e);
    welfareexstrong(i) = welfare(cexstrong,lexstrong,-demoexstrong,mass,r,My,e);
    
    % compute lump sum increase equivalent to welfare gain from optimal tax
    % absolute change in lump sum payment:
    cwelfaregaintoexlumpabs(i) = fsolve(@(add) 1000000000*(welfare(cex+add,lex,-demoex+add,mass,r,My,e)-welfareopt(i)),1);
    cwelfaregaintoexlumpabsstrong(i) = fsolve(@(add) 1000000000*(welfare(cexstrong+add,lexstrong,-demoexstrong+add,mass,r,My,e)-welfareoptstrong(i)),1);
    % relative to average income at exo tech optimum:
    cwelfaregaintoexlumpgdp(i) = cwelfaregaintoexlumpabs(i)/ymeanex;
    cwelfaregaintoexlumpgdpstrong(i) = cwelfaregaintoexlumpabsstrong(i)/ymeanexstrong;
    % relative to lump sum payment at exo tech optimum:
    cwelfaregaintoexlump (i) = cwelfaregaintoexlumpabs(i)/(-demoex);
    cwelfaregaintoexlumpstrong (i) = cwelfaregaintoexlumpabsstrong(i)/(-demoexstrong);
end